/*** Part of the ICMP monitor. (C) 1999/2000 by Stealth. GPL.
 ***
 ***/
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include "imon.h"

extern int promisc;

void
inthandler(int sig)
{
#ifdef REENTRANT
	printf("\nCaught signal %d (should be CTRL-C).\n", sig);
#endif
	if (promisc)
		droppromisc(promisc);
	/* we pushed the fd in promisc in main() */
	close(promisc);
	exit(0);
}

/* If the type or code is out of range the 'was' should be "" */
void
bounds_check(u_int8_t *type, u_int8_t *code)
{
   	if (*type >= NR_TYPE || *code >= NR_CODE) {
           	printf("badly formed ICMP packet (type=%d, code=%d)\n", *type, *code);
           	*type = 1;
                *code = 1;
        }
}

void
help_her(const char *s)
{
   	puts(we);
	fprintf(stderr, "\nusage:\n"
			" %s [-i intf] [-o file] [-L host] [-l time] [-rsdg]\n\n"
                        "     -i intf  -  use this interface\n"
                        "     -o file  -  force output to this file (daemon mode!)\n"
                        "     -r       -  resolve IP adresses to hostnames\n"
                        "     -s       -  print out the sequence and id field\n"
			"     -d       -  drop ECHO/ECHO_REPLY type\n"
			"     -l time  -  passive LOKI detect within 'time' seconds\n"
			"                 (100 gives a reliable result)\n"   
			"     -L host  -  active LOKI detection; scan 'host'\n" 
#ifdef GPL_IN
			"     -g       -  show license.\n"
#endif
			"\n", s);
        exit(1);
}


/* if how == 0, only give back IP */
char*
resolv(unsigned int addr, int how)
{
	    char *s;
	    struct hostent *he;
	    struct in_addr in;
	    
	    in.s_addr = addr;
	    if ((s = (char*)malloc(500)) == NULL) {
		    perror("malloc");
		    exit(errno);
	    }
	    if (how == 0 || (he = gethostbyaddr((const char*)&in, sizeof(in), AF_INET)) == NULL) 
		    strncpy(s, inet_ntoa(in), 400);
	    else
		    strncpy(s, he->h_name, 400);
	    return s;
}
	
		    
		    
/* It seems that nobody, except the 'ping' programmer
 * has ever written a own function for this. Me too... ;-)
 */
unsigned short
in_cksum (unsigned short *ptr, int nbytes)
{

  register long sum;		/* assumes long == 32 bits */
  u_short oddbyte;
  register u_short answer;	/* assumes u_short == 16 bits */

  /*
   * Our algorithm is simple, using a 32-bit accumulator (sum),
   * we add sequential 16-bit words to it, and at the end, fold back
   * all the carry bits from the top 16 bits into the lower 16 bits.
   */

  sum = 0;
  while (nbytes > 1)
    {
      sum += *ptr++;
      nbytes -= 2;
    }

  /* mop up an odd byte, if necessary */
  if (nbytes == 1)
    {
      oddbyte = 0;		/* make sure top half is zero */
      *((u_char *) & oddbyte) = *(u_char *) ptr;	/* one byte only */
      sum += oddbyte;
    }

  /*
   * Add back carry outs from top 16 bits to low 16 bits.
   */

  sum = (sum >> 16) + (sum & 0xffff);	/* add high-16 to low-16 */
  sum += (sum >> 16);		/* add carry */
  answer = ~sum;		/* ones-complement, then truncate to 16 bits */
  return (answer);
}

